"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// Modules to control application life and create native browser window
const electron_1 = require("electron");
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const file_url_1 = __importDefault(require("file-url"));
class SearchView {
    constructor(window) {
        this.view = null;
        this.lastSearch = "";
        this.allowFindNext = false;
        this.onWindowClosed = () => {
            this.window = null;
            this.view = null;
            SearchView.list = SearchView.list.filter((e) => e !== this);
        };
        this.reposition = () => {
            if (this.window && this.view) {
                const bounds = this.window.getContentBounds();
                const width = 400;
                const height = 35;
                this.view.setBounds({ x: 0, y: bounds.height - height, width, height });
            }
        };
        this.show = () => {
            if (this.window && !this.view) {
                this.view = new electron_1.BrowserView({
                    webPreferences: {
                        contextIsolation: true,
                        preload: path_1.default.join(__dirname, "searchPreload.js"),
                    },
                });
                this.window.setBrowserView(this.view);
                this.view.webContents.loadFile(path_1.default.join(__dirname, "search.html"));
                this.view.webContents.loadURL(`${(0, file_url_1.default)(path_1.default.join(__dirname, "search.html"))}#${this.lastSearch}`);
                this.reposition();
                this.view.webContents.focus();
            }
        };
        this.hide = () => {
            if (this.window && this.view) {
                this.window.removeBrowserView(this.view);
                this.view = null;
                this.stopFindInPage("keepSelection");
            }
        };
        this.found = (event, result) => {
            if (this.window && this.view)
                this.view.webContents.send("update", result.activeMatchOrdinal, result.matches);
        };
        this.window = window;
        SearchView.list.push(this);
        window.on("closed", this.onWindowClosed);
        window.on("resize", this.reposition);
        window.webContents.on("found-in-page", this.found);
    }
    static fromBrowserWindow(window) {
        var _a;
        return (_a = SearchView.list.find((e) => e.window === window)) !== null && _a !== void 0 ? _a : null;
    }
    static fromWebContents(webContents) {
        var _a;
        return (_a = SearchView.list.find((e) => { var _a; return ((_a = e.view) === null || _a === void 0 ? void 0 : _a.webContents) === webContents; })) !== null && _a !== void 0 ? _a : null;
    }
    findInPage(text, options) {
        if (this.window) {
            this.lastSearch = text;
            this.window.webContents.findInPage(text, options);
            this.allowFindNext = true;
        }
    }
    stopFindInPage(action) {
        if (this.window) {
            this.window.webContents.stopFindInPage(action);
            this.allowFindNext = false;
        }
    }
    findNext(forward) {
        if (this.lastSearch)
            this.findInPage(this.lastSearch, { findNext: this.allowFindNext, forward });
        else
            this.show();
    }
}
SearchView.list = [];
function createWindow(workingDirectory) {
    let win = new electron_1.BrowserWindow({
        width: 800,
        height: 600,
        minWidth: 800,
        minHeight: 600,
        webPreferences: {
            contextIsolation: true,
            preload: path_1.default.join(__dirname, "preload.js"),
        },
    });
    if (!electron_1.app.commandLine.hasSwitch("debug"))
        win.setMenuBarVisibility(false);
    win.loadURL(`${(0, file_url_1.default)(path_1.default.join(__dirname, "docs", "index.html"))}#${workingDirectory}`);
    const searchView = new SearchView(win);
    win.on("closed", () => {
        win = null;
    });
    win.on("focus", () => {
        electron_1.globalShortcut.register("CommandOrControl+F", searchView.show);
        electron_1.globalShortcut.register("F3", () => searchView.findNext(true));
        electron_1.globalShortcut.register("Shift+F3", () => searchView.findNext(false));
    });
    win.on("blur", () => {
        electron_1.globalShortcut.unregisterAll();
    });
}
electron_1.app.whenReady().then(() => createWindow(process.cwd()));
electron_1.app.on("window-all-closed", () => {
    process.platform !== "darwin" && electron_1.app.quit();
    electron_1.globalShortcut.unregisterAll();
});
electron_1.app.on("activate", () => {
    electron_1.BrowserWindow.getAllWindows().length === 0 && createWindow(process.cwd());
});
function on(channel, listener) {
    electron_1.ipcMain.on(channel, (event, ...args) => {
        event.returnValue = listener(event, ...args);
    });
}
function onSearch(channel, listener) {
    on(channel, (event, ...args) => {
        const search = SearchView.fromWebContents(event.sender);
        search && listener(search, event, ...args);
    });
}
onSearch("find-in-page", (search, _event, text, options) => search.findInPage(text, options));
onSearch("stop-find-in-page", (search, _event, action) => search.stopFindInPage(action));
onSearch("hide-search", (search) => search.hide());
on("close", () => electron_1.dialog.showMessageBoxSync({
    type: "question",
    title: "Files have not been saved",
    message: "There are unsaved changes. Discard these changes?",
    buttons: ["OK", "Cancel"],
    defaultId: 1,
    cancelId: 1,
}) === 0);
on("open-directory", (event) => {
    const window = electron_1.BrowserWindow.fromWebContents(event.sender);
    if (window) {
        electron_1.dialog
            .showOpenDialog(window, {
            properties: ["openDirectory"],
            title: "Select your Web Extension root directory",
        })
            .then((result) => {
            if (!result.canceled && result.filePaths.length)
                createWindow(result.filePaths[0]);
        });
    }
});
const isDirectory = (file) => fs_1.default.existsSync(file) && fs_1.default.lstatSync(file).isDirectory();
const isFile = (file) => fs_1.default.existsSync(file) && !fs_1.default.lstatSync(file).isDirectory();
const readFile = (file) => fs_1.default.readFileSync(file, "utf-8");
on("load-files", (event, extDir) => {
    const localesDir = path_1.default.join(extDir, "_locales");
    const manifestFile = path_1.default.join(extDir, "manifest.json");
    if (isFile(manifestFile) && isDirectory(localesDir)) {
        try {
            const editorConfigsToLoad = new Set();
            const locales = [];
            for (const localeDir of fs_1.default.readdirSync(localesDir)) {
                const localePath = path_1.default.join(localesDir, localeDir);
                if (!isDirectory(localePath))
                    continue;
                const messagesPath = path_1.default.join(localePath, "messages.json");
                if (isFile(messagesPath)) {
                    const editorConfigPaths = [
                        [".editorconfig"],
                        ["..", ".editorconfig"],
                        ["..", "..", ".editorconfig"],
                    ]
                        .map((paths) => path_1.default.join(localePath, ...paths))
                        .filter(isFile);
                    editorConfigPaths.forEach((p) => editorConfigsToLoad.add(p));
                    locales.push({
                        path: messagesPath,
                        data: readFile(messagesPath),
                        locale: localeDir.replace("_", "-"),
                        editorConfigs: editorConfigPaths,
                    });
                }
            }
            const editorConfigs = [...editorConfigsToLoad.values()].map((editorConfigPath) => ({
                path: editorConfigPath,
                data: readFile(editorConfigPath),
            }));
            return {
                locales,
                manifest: {
                    path: manifestFile,
                    data: readFile(manifestFile),
                },
                editorConfigs,
            };
        }
        catch (e) {
            return String(e);
        }
    }
    return false;
});
on("save-files", (event, extDir, files) => {
    const localesDir = path_1.default.join(extDir, "_locales");
    if (isDirectory(localesDir)) {
        try {
            for (const { locale, data } of files) {
                const dir = path_1.default.join(localesDir, locale.replace("-", "_"));
                if (!isDirectory(dir))
                    fs_1.default.mkdirSync(dir);
                const file = path_1.default.join(dir, "messages.json");
                fs_1.default.writeFileSync(file, data);
            }
            return null;
        }
        catch (e) {
            return String(e);
        }
    }
    return "Directory does not exist";
});
