"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const electron_1 = require("electron");
electron_1.contextBridge.exposeInMainWorld("electronBridge", {
    init() {
        const search = document.getElementById("search");
        const label = document.getElementById("label");
        const prev = document.getElementById("prev");
        const next = document.getElementById("next");
        const close = document.getElementById("close");
        search.focus();
        if (document.location.hash.length > 1)
            search.value = document.location.hash.substr(1);
        else {
            prev.setAttribute("disabled", "disabled");
            next.setAttribute("disabled", "disabled");
        }
        // fixme:
        // matchCase?: boolean;
        // wordStart?: boolean;
        function on(target, ev, cb) {
            target.addEventListener(ev, cb);
        }
        on(search, "input", () => {
            if (search.value) {
                electron_1.ipcRenderer.sendSync("find-in-page", search.value);
                search.focus();
                prev.removeAttribute("disabled");
                next.removeAttribute("disabled");
            }
            else {
                electron_1.ipcRenderer.sendSync("stop-find-in-page", "clearSelection");
                label.innerText = "";
                prev.setAttribute("disabled", "disabled");
                next.setAttribute("disabled", "disabled");
            }
        });
        on(search, "keyup", (e) => {
            if (e.key === "Escape")
                electron_1.ipcRenderer.sendSync("hide-search", search.value);
            else if (search.value && e.key === "Enter") {
                electron_1.ipcRenderer.sendSync("find-in-page", search.value, { forward: !e.shiftKey, findNext: true });
                search.focus();
            }
        });
        on(prev, "click", () => {
            search.focus();
            electron_1.ipcRenderer.sendSync("find-in-page", search.value, { forward: false, findNext: true });
        });
        on(next, "click", () => {
            search.focus();
            electron_1.ipcRenderer.sendSync("find-in-page", search.value, { findNext: true });
        });
        on(close, "click", () => {
            electron_1.ipcRenderer.sendSync("hide-search", search.value);
        });
        electron_1.ipcRenderer.on("update", (_, index, count) => {
            if (!search.value.length)
                return;
            if (count === null)
                label.innerHTML = "";
            else
                label.innerText = `${index}/${count}`;
        });
        on(window, "focus", () => search.focus());
    },
});
